@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: February 2018 */
+/* This file last modified: January 2019 */
 
 /* This is the header file that contains definitions of the structures used for
 holding musical data, and various flags and enumerations. */
@@ -191,29 +191,25 @@ useful. */
 
 enum { ac_none, ac_dsharp, ac_flat, ac_dflat, ac_natural, ac_sharp };
 
-/* Type values that identify each structure in a bar's data */
+/* Type values that identify each structure in a bar's data. This list of types 
+must be kept in step with the table of structure lengths called length_table in 
+src/tables.c. */
 
 enum {
   b_End,      /* End of bar */
   b_Jump,     /* Connection to another block of store */
 
-  b_all,            b_barnum,    b_beamacc,      b_beambreak, b_beambreak2,
-  b_beamrit,        b_bowing,    b_breakbarline, b_caesura,   b_chord,
-  b_clef,           b_comma,     b_dbar,         b_dotbar,    b_dotright,
-  b_draw,           b_dynmove,   b_ebar,         b_endplet,   b_endslur,   
-  b_ens,
-  b_ensure,         b_footnote,  b_hairpin,      b_ibar,      b_justify,
-  b_key,            b_linegap,   b_lrepeat,      b_masq,      b_move,
-  b_name,           b_nbar,      b_newline,      b_newpage,   b_nopack,
-  b_note,           b_noteheads, b_notes,        b_ns,        b_nsm,
-  b_offset,         b_olevel,    b_olhere,       b_ornament,  b_page,
-  b_pagebots,       b_pagetops,  b_playchange,   b_plet,      b_prevbar,
-  b_reset,          b_resume,    b_rrepeat,      b_setclef,   b_setkey,
-  b_settime,        b_sghere,    b_sgnext,       b_slope,     b_slur,
-  b_slurgap,        b_slurmod,   b_space,        b_sshere,    b_ssnext,
-  b_suspend,        b_text,      b_textX,        b_tick,      b_tie,
-  b_time,           b_tremolo,   b_tripsw,       b_ulevel,    b_ulhere,
-  b_unbreakbarline, b_zcopy,
+  b_all, b_barnum, b_beamacc, b_beambreak, b_beambreak2, b_beamrit, b_bowing,
+  b_breakbarline, b_caesura, b_chord, b_clef, b_comma, b_dbar, b_dotbar,
+  b_dotright, b_draw, b_dynmove, b_ebar, b_endplet, b_endslur, b_ens, b_ensure,
+  b_footnote, b_hairpin, b_ibar, b_justify, b_key, b_linegap, b_lrepeat,
+  b_masq, b_move, b_name, b_nbar, b_newline, b_newpage, b_nopack, b_note,
+  b_noteheads, b_notes, b_ns, b_nsm, b_offset, b_olevel, b_olhere, b_ornament,
+  b_page, b_pagebots, b_pagetops, b_playchange, b_plet, b_prevbar, b_reset,
+  b_resume, b_rrepeat, b_setclef, b_setkey, b_settime, b_sgabove, b_sghere,
+  b_sgnext, b_slope, b_slur, b_slurgap, b_slurmod, b_space, b_sshere, b_ssnext,
+  b_suspend, b_text, b_textX, b_tick, b_tie, b_time, b_tremolo, b_tripsw,
+  b_ulevel, b_ulhere, b_unbreakbarline, b_zcopy,
 
   b_baditem   /* For detecting bad values */
 };
