@@ -705,14 +705,14 @@ static void debug_heading(uschar *s, headstr *p)
 {
 while (p != NULL)
   {
-  if (p->size == -2)
+  if (p->size < 0)
     {
-    tree_node *t = p->a.drawing;
+    tree_node *t = p->drawing;
     debug_printf("%s draw %s %f\n", s, t->name, p->space);
     }
   else
     {
-    debug_printf("%s %f \"%s\" %f\n", s, p->size, p->a.text, p->space);
+    debug_printf("%s %f \"%s\" %f\n", s, p->size, p->text, p->space);
     }
   p = p->next;
   }
