@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 2nd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: January 2019 */
 
 
 /* This file contains code for handling errors. There are some spares in the
@@ -39,7 +39,7 @@ static error_struct error_data[] = {
 { rc_serious,  "Mis-placed %s (does not immediately follow a note)\n" },
 { rc_serious,  "Wrong data type on stack for \"%s\" while drawing \"%s\" %s\n" },
 { rc_disaster, "Unable to open font file \"%s\": %s (check PMW installation)\n" },
-{ rc_serious,  "Defined name expected after \"&\"\n" },
+{ rc_serious,  "Macro name or string repetition expected after \"&\"\n" },
 { rc_serious,  "The name \"%s\" has not been defined\n" },
 /* 10-14 */
 { rc_serious,  "%s expected" },
@@ -84,7 +84,7 @@ static error_struct error_data[] = {
 { rc_serious,  "A chord may not contain a rest\n" },
 { rc_serious,  "The notes of a chord must all be the same length\n" },
 /* 45-49 */
-{ rc_serious,  "Error in note expression or ornament data\n" },
+{ rc_serious,  "Error in note expression or ornament\n" },
 { rc_serious,  "Conflicting stem direction requests %s\n" },
 { rc_serious,  "The \\sw\\ option is only available for beamed notes when the\n"
                "   stem direction of the first note is forced\n" },
@@ -136,7 +136,7 @@ static error_struct error_data[] = {
 { rc_serious,  "Malformed page list on command line\n" },
 { rc_disaster, "Page range out of order on command line\n" },
 { rc_disaster, "Malformed -dsb option data\n" },
-{ rc_warning,  "Character value %d is greater than 255 for a non-standardly encoded\n" 
+{ rc_warning,  "Character value %d is greater than 255 for a non-standardly encoded\n"
                "   font - changed to %d\n" },
 { rc_warning,  "Unsupported Unicode code point U+%04X changed to U+%04X\n" },
 /* 80-84 */
@@ -209,10 +209,17 @@ static error_struct error_data[] = {
 /* 130-134 */
 { rc_disaster, "PostScript font \"%s\" not found in \"%s\" or \"%s\"\n" },
 { rc_disaster, "Too many text strings before a note (%d maximum)\n" },
-{ rc_disaster, "Too many notes in a chord (%d maximum)\n" }
+{ rc_disaster, "Too many notes in a chord (%d maximum)\n" },
+{ rc_serious,  "Missing backslash after note options" },
+{ rc_serious,  "A &* replication must have only one argument - others ignored\n" },
+/* 135-139 */
+{ rc_serious,  "No options are allowed after \"x\" note repetition" },
+{ rc_disaster, "Macro %s is too long (max %d)\n" },
+{ rc_disaster, "Too many macro arguments (max %d)\n" },
+{ rc_warning,  "Warning: %s repeat at %s of bar - misplaced bar line?\n" }
 };
 
-#define error_maxerror 132   /* Tests are in misctests/Errors */
+#define error_maxerror 138   /* Tests are in misctests/Errors */
 
 /* Save some typing */
 
@@ -235,7 +242,7 @@ void
 error_moan(int n, ...)
 {
 int rc;
-int oldchptr;
+BOOL do_skip = FALSE;
 uschar buff[256];
 va_list ap;
 va_start(ap, n);
@@ -261,37 +268,37 @@ else
   rc = error_data[n].rc;
   }
 
-/* Save character position for marking input line reflection */
+/* We skip to the end of the input line or ']' or '\' for certain errors. Add
+info to the error message here; the actual skip happens later after the message
+is output (so that the right position in the input is shown). */
 
-oldchptr = read_chptr - this_buffer - 1;
-
-/* We skip to the end of the input line or ']' or '\' for certain errors.
-However, we don't skip to EOL if we are in fact at the start of a line, since
-that case must really be a mistake in the previous line. */
-
-if (n == 10 || n == 19 || n == 20 || n == 32) switch(error_skip)
+if (n == 10 || n == 19 || n == 20 || n == 32 || n == 133 || n == 135)
   {
-  case skip_EOL:
-  if (read_chptr != this_buffer + 1)
+  do_skip = TRUE;
+  switch(error_skip)
     {
-    Ustrcat(buff, " - skipping to end of line\n");
-    read_chptr = read_endptr;
-    read_ch = ' ';
+    case skip_EOL:
+    Ustrcat(buff, " - skipping to end of line");
+    break;
+
+    case skip_KET:
+    Ustrcat(buff, " - skipping to next \"]\" or end of line");
+    break;
+
+    case skip_BACKSLASH:
+    Ustrcat(buff, " - skipping to next \"\\\" or end of line");
+    break;
+
+    case skip_BAR:
+    Ustrcat(buff, " - skipping to next \"|\" or end of line");
+    stave_checklength = FALSE;   /* Don't check bar length */ 
+    break;
+
+    default:
+    break;
     }
-  else Ustrcat(buff, "\n");
-  break;
 
-  case skip_KET:
-  Ustrcat(buff, " - skipping to next \"]\" or end of line\n");
-  while (read_ch != ']' && read_ch != '\n' && read_ch != EOF) next_ch();
-  sigch();
-  break;
-
-  default:
-  Ustrcat(buff, " - skipping to next \"\\\" or end of line\n");
-  while (read_ch != '\\' && read_ch != '\n' && read_ch != EOF) next_ch();
-  sigch();
-  break;
+  Ustrcat(buff, "\n");
   }
 
 /* Display the message */
@@ -303,7 +310,7 @@ eprintf(buff);
 if (!main_initialized) rc = rc_disaster;
 
 /* If the line number is non-zero, show it, but only in the reading state. We
-used to add the file name only for included files, but now it is always 
+used to add the file name only for included files, but now it is always
 included for the benefit of people who use the "compilation mode" of Emacs (or
 similar feature in other environments). This makes use of the error messages
 and is able to open the file at the place where the error was found during the
@@ -313,12 +320,12 @@ if (reading_input)
   {
   if (read_linenumber > 0)
     {
-    uschar *name = (main_filename == NULL)? US"<stdin>" : main_filename; 
+    uschar *name = (main_filename == NULL)? US"<stdin>" : main_filename;
     if (read_filestackptr > 0)
-      sprintf(CS buff, "** File \"%s\" (included), near line %d", name, 
+      sprintf(CS buff, "** File \"%s\" (included), near line %d", name,
         read_linenumber);
-    else 
-      sprintf(CS buff, "** File \"%s\", near line %d", name, 
+    else
+      sprintf(CS buff, "** File \"%s\", near line %d", name,
         read_linenumber);
     eprintf(buff);
     }
@@ -331,6 +338,7 @@ if (reading_input)
 
   if (this_buffer[0] != 0 || prev_buffer[0] != 0)
     {
+    int oldchptr = read_chptr - this_buffer - 1;
     if (error_ptr_correction <= oldchptr) oldchptr -= error_ptr_correction;
     error_ptr_correction = 0;
     eprintf(US":\n");
@@ -356,6 +364,35 @@ if (reading_input)
 
 eprintf(US"\n");
 
+/* Skip to the end of the input line or ']' or '\' for certain errors (detected
+above, when the message was modified). */
+
+if (do_skip) switch(error_skip)
+  {
+  case skip_EOL:
+  read_chptr = read_endptr;       
+  read_ch = ' ';
+  break;                    
+ 
+  case skip_KET:
+  while (read_ch != ']' && read_ch != '\n' && read_ch != EOF) next_ch();
+  sigch();
+  break;
+
+  case skip_BACKSLASH:
+  while (read_ch != '\\' && read_ch != '\n' && read_ch != EOF) next_ch();
+  sigch();
+  break;
+  
+  case skip_BAR:
+  while (read_ch != '|' && read_ch != '\n' && read_ch != EOF) next_ch();
+  sigch();
+  break;
+  
+  default:
+  break;  
+  }
+
 /* Disaster if too many errors */
 
 if (rc > rc_warning)
