@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: January 2019 */
 
 
 /* This file contains code for miscellanous functions used when paginating,
@@ -39,29 +39,29 @@ static int resttable[] = {
 
 /* This function takes an integer value in the range 0 - 0x7fffffff
 and encodes it as a UTF-8 character in 0 to 6 bytes.
-                                                  
-Arguments:                                        
-  cvalue     the character value                  
+
+Arguments:
+  cvalue     the character value
   buffer     pointer to buffer for result - at least 6 bytes long
-                                                   
+
 Returns:     number of characters placed in the buffer
-*/                        
-                    
+*/
+
 int
 misc_ord2utf8(int cvalue, uschar *buffer)
-{     
-register int i, j;   
+{
+register int i, j;
 for (i = 0; i < 6; i++) if (cvalue <= utf8_table1[i]) break;
-buffer += i;               
-for (j = i; j > 0; j--)                                    
-  {                  
-  *buffer-- = 0x80 | (cvalue & 0x3f);                   
+buffer += i;
+for (j = i; j > 0; j--)
+  {
+  *buffer-- = 0x80 | (cvalue & 0x3f);
   cvalue >>= 6;
   }
 *buffer = utf8_table2[i] | cvalue;
 return i + 1;
 }
-                                       
+
 
 
 /*************************************************
@@ -70,17 +70,17 @@ return i + 1;
 
 /* This function checks for a special string for the key signature; otherwise
 it finds the number of accidentals in a key signature, and returns an
-appropriate printing width. In both cases this is the width for a 10-point 
+appropriate printing width. In both cases this is the width for a 10-point
 stave (appropriate adjustment happens in the calling function when needed).
 
-Arguments:   
+Arguments:
   key       the key signature
   clef      the clef - used only to check for a special
-    
+
 Returns:    the printing width
 */
 
-int 
+int
 misc_keywidth(int key, int clef)
 {
 pkeystr *pk;
@@ -90,26 +90,26 @@ int key63 = key & 63;
 DEBUG(("misc_keywidth() key=%d clef=%d\n", key, clef));
 
 for (pk = main_printkey; pk != NULL; pk = pk->next)
-  { 
-  if (key63 == pk->key && clef == pk->clef && 
-      pk->movt_number <= curmovt->number) 
-    break; 
+  {
+  if (key63 == pk->key && clef == pk->clef &&
+      pk->movt_number <= curmovt->number)
+    break;
   }
-   
+
 if (pk != NULL)
   {
   width = string_width((key>63)? pk->cstring : pk->string, font_mf, 10000);
   }
-   
+
 else
-  {       
+  {
   int n = main_keysigtable[key63];
   width = abs(n) *
     (curmovt->accspacing)[(key>63)? ac_natural : (n>0)?ac_sharp : ac_flat];
-  }   
-   
+  }
+
 DEBUG(("misc_keywidth() = %d\n", width));
-return width; 
+return width;
 }
 
 
@@ -118,14 +118,14 @@ return width;
 *          Return width of time signature        *
 *************************************************/
 
-/* This function sets up the appropriate strings for a time signature and then 
+/* This function sets up the appropriate strings for a time signature and then
 returns the width of the longer one.
 
 Argument:    the time signature
 Returns:     the printing width
 */
 
-int 
+int
 misc_timewidth(int ts)
 {
 ptimestr *pt = main_printtime;
@@ -141,7 +141,7 @@ uschar *botstring = vd;
 if (!curmovt->showtime) return 0;
 
 /* First see if this time signature has special strings specified for its
-printing. The specification must have happened in this movement or earlier to 
+printing. The specification must have happened in this movement or earlier to
 be applicable. */
 
 while (pt != NULL)
@@ -198,13 +198,13 @@ return (yieldn > yieldd)? yieldn : yieldd;
 
 /* If the second argument is not null, we use it to return any
 ornament value. We always advance by at least one item. The initial
-item need not be a note. Take care in case it is a Jump or End. 
+item need not be a note. Take care in case it is a Jump or End.
 
 Arguments:
   p         pointer to the current item
   orn       if not NULL, return the ornament that preceded the note
-  
-Returns:    pointer to the next note item or NULL if there isn't one 
+
+Returns:    pointer to the next note item or NULL if there isn't one
 */
 
 b_notestr *
@@ -226,7 +226,7 @@ for (;;)
     if (!beam_overbeam) return NULL;  /* a barline, continue past the end. */
     break;                            /* There will always be another bar. */
 
-    case b_note: 
+    case b_note:
     return p;
 
     case b_ornament:
@@ -234,7 +234,7 @@ for (;;)
     break;
 
     case b_Jump:
-    p = (b_notestr *)(((b_Jumpstr *)p)->next);                      
+    p = (b_notestr *)(((b_Jumpstr *)p)->next);
     break;
     }
 
@@ -257,7 +257,7 @@ incslur is passed as TRUE, causing the section numbers of slurs to be
 increased. (At the start of the first system, it is FALSE.) The result of the
 function is a pointer to a vector of contstr structures, one for each stave.
 
-Arguments:   
+Arguments:
   p          pointer to the current contstr
   count      the highest stave number that we need to save (can be 0)
   incslur    when TRUE, increment the section number for each slur
@@ -329,9 +329,9 @@ for (i = 0; i <= count; i++)
     slurstr *ss = store_Xget(sizeof(slurstr));
     gapstr *g = s->gaps;
     gapstr **gprev = &(ss->gaps);
-    if (incslur) s->section += 1; 
+    if (incslur) s->section += 1;
     ss[0] = s[0];
- 
+
     while (g != NULL)
       {
       gapstr *gg = store_Xget(sizeof(gapstr));
@@ -373,16 +373,16 @@ return yield;
 *         Free a continued data structure        *
 *************************************************/
 
-/* This is called at the end of outputting a system 
+/* This is called at the end of outputting a system
 
 Arguments:
   p         pointer to the contstr vector
   count     number of staves saved
-  
-Returns:    nothing  
+
+Returns:    nothing
 */
 
-void 
+void
 misc_freecontstr(contstr *p, int count)
 {
 int i;
@@ -446,13 +446,13 @@ store_free(p);
 
 /* Certain items require the same actions when encountered in the advance scan
 of each system by page_setcont() as when encountered while actually outputting
-a system. These actions are handled in this function. 
+a system. These actions are handled in this function.
 
 Argument:    pointer to the item
 Returns:     nothing
 */
 
-void 
+void
 misc_commoncont(bstr *p)
 {
 switch(p->type)
@@ -510,7 +510,7 @@ Argument:   nothing
 Returns:    TRUE if there is pending underlay
 */
 
-static BOOL 
+static BOOL
 pendulay(void)
 {
 ulaystr *u = bar_cont->ulay;
@@ -523,12 +523,12 @@ return FALSE;
 }
 
 
-/* 
+/*
 Argument:   nothing
 Returns:    TRUE if there is pending overlay
 */
 
-static BOOL 
+static BOOL
 pendolay(void)
 {
 ulaystr *u = bar_cont->ulay;
@@ -546,16 +546,16 @@ return FALSE;
 *************************************************/
 
 /* This function is called from page_setcont(). We have to flag the note and
-not do it twice, to cope with repeated bars. 
+not do it twice, to cope with repeated bars.
 
 Arguments:
   p            the note
   upflag       TRUE if the stem is up
-  
-Returns:       nothing  
+
+Returns:       nothing
 */
 
-static void 
+static void
 shorten_stem(b_notestr *tp, int upflag)
 {
 int shorten = 0;
@@ -568,13 +568,13 @@ else
   {
   if (tp->spitch < 136) shorten = 250*(136 - tp->spitch);
   }
-  
+
 if (shorten != 0 && (tp->flags & nf_shortened) == 0)
-  { 
+  {
   tp->yextra -= (shorten <= curmovt->shorten)? shorten : curmovt->shorten;
-  tp->flags |= nf_shortened; 
-  } 
-}   
+  tp->flags |= nf_shortened;
+  }
+}
 
 
 
@@ -603,7 +603,7 @@ Arguments:  none
 Returns:    nothing
 */
 
-void 
+void
 page_setcont(void)
 {
 BOOL hadssnext = FALSE;
@@ -644,7 +644,7 @@ for (stave = 0; stave <= page_lastwanted; stave++)
           switch(type)
             {
             case b_Jump:
-            p = (bstr *)(((b_Jumpstr *)p)->next);                      
+            p = (bstr *)(((b_Jumpstr *)p)->next);
             break;
 
             case b_End:
@@ -672,27 +672,27 @@ for (stave = 0; stave <= page_lastwanted; stave++)
               (void)out_setupbeam(beamfirst, beammoff, TRUE, FALSE);
               }
             break;
-            
+
             case b_footnote:
               {
               headstr *h = &(((b_footnotestr *)p)->h);
               headstr *hh = h;
-              headstr *lh = NULL; 
+              headstr *lh = NULL;
               page_justifyheading(h);
-              while (hh != NULL) 
+              while (hh != NULL)
                 {
-                lh = hh; 
+                lh = hh;
                 page_newfootnotedepth += hh->space;
                 hh = hh->next;
-                }  
+                }
               if (page_newfootnotes == NULL) page_newfootnotes = h; else
-                { 
+                {
                 page_lastnewfootnote->next = h;
-                h->b.spaceabove = curmovt->footnotesep;
-                }  
+                h->spaceabove = curmovt->footnotesep;
+                }
               page_lastnewfootnote = lh;
               }
-            break;    
+            break;
 
             case b_slur:
             (void)misc_setstartslur(p);
@@ -759,7 +759,7 @@ for (stave = 0; stave <= page_lastwanted; stave++)
               {
               b_notestr *tp = (b_notestr *)p;
               int flags = tp->flags;
-              int acflags = tp->acflags; 
+              int acflags = tp->acflags;
               int apitch = tp->spitch;
               int bpitch = tp->spitch;
               int acc = tp->acc;
@@ -780,10 +780,10 @@ for (stave = 0; stave <= page_lastwanted; stave++)
                 }
 
               upflag = (flags & nf_stemup) != 0;
-              if (curmovt->shorten && tp->length != 0) 
+              if (curmovt->shorten && tp->length != 0)
                 shorten_stem(tp, upflag);
-               
-              stemlength = tp->yextra; 
+
+              stemlength = tp->yextra;
               chordcount = 1;
               mac_advancechord(tp);
 
@@ -791,7 +791,7 @@ for (stave = 0; stave <= page_lastwanted; stave++)
                 {
                 chordcount++;
                 flags |= tp->flags;
-                acflags |= tp->acflags; 
+                acflags |= tp->acflags;
                 if (tp->spitch > apitch)
                   {
                   apitch = tp->spitch;
@@ -802,7 +802,7 @@ for (stave = 0; stave <= page_lastwanted; stave++)
                   bpitch = tp->spitch;
                   acc = tp->acc;
                   }
-                if (curmovt->shorten && tp->length != 0) 
+                if (curmovt->shorten && tp->length != 0)
                   shorten_stem(tp, upflag);
                 if (abs(tp->yextra) > abs(stemlength)) stemlength = tp->yextra;
                 mac_advancechord(tp);
@@ -884,12 +884,12 @@ for (stave = 0; stave <= page_lastwanted; stave++)
                 else uu = &(u->next);
                 u = *uu;
                 }
- 
+
               lastAlevel = lastBlevel = 0;
               if (((b_notestr *)p)->notetype < quaver) beamfirst = NULL;
               }
-              
-            /* Notes and rests dealt with */ 
+
+            /* Notes and rests dealt with */
 
             moff += ((b_notestr *)p)->length;
             hadulay = hadolay = FALSE;
@@ -968,6 +968,24 @@ for (stave = 0; stave <= page_lastwanted; stave++)
             bar_cont->key = ((b_keystr *)p)->key;
             break;
 
+            case b_sgabove:            /* retrospective change of system gap */
+              {
+              b_sgstr *pp = (b_sgstr *)p;
+              int v = pp->value;
+              sysblock *pb = curpage->sysblocks;
+              if (pb != NULL)  /* There may be a previous system */
+                {
+                while (pb->next != NULL) pb = pb->next;  /* Find last block */
+                if(pb->type == sh_system)                /* Ignore if heading */
+                  {
+                  curpage->spaceleft += pb->systemgap; 
+                  if (pp->opt == '+') pb->systemgap += v;
+                    else pb->systemgap = v;
+                  curpage->spaceleft -= pb->systemgap; 
+                  }
+                }
+              }
+            break;
 
             case b_sghere:            /* immediate change of system gap */
               {
@@ -986,7 +1004,6 @@ for (stave = 0; stave <= page_lastwanted; stave++)
               }
             break;
 
-
             case b_sshere:            /* immediate change of stave spacing */
               {
               b_ssstr *pp = (b_ssstr *)p;
@@ -1005,11 +1022,11 @@ for (stave = 0; stave <= page_lastwanted; stave++)
                 int i;
                 for (i = 1; i < page_lastwanted; i++)
                   {
-                  if (pp->opt == '+')  
+                  if (pp->opt == '+')
                     page_sysblock->stavespacing[i] += v;
-                  else   
+                  else
                     page_sysblock->stavespacing[i] = v;
-                  }   
+                  }
                 }
               else if (pp->opt == '+')
                 {
@@ -1044,13 +1061,13 @@ for (stave = 0; stave <= page_lastwanted; stave++)
               if (s == 0)
                 {
                 int i;
-                for (i = 1; i < page_lastwanted; i++) 
+                for (i = 1; i < page_lastwanted; i++)
                   {
-                  if (pp->opt == '+')  
+                  if (pp->opt == '+')
                     page_ssnext[i] += v;
-                  else   
+                  else
                     page_ssnext[i] = v;
-                  }   
+                  }
                 }
               else if (pp->opt == '+') page_ssnext[s] += v;
                 else page_ssnext[s] = v;
@@ -1088,12 +1105,12 @@ for (stave = 0; stave <= page_lastwanted; stave++)
               b_pagestr *pg = (b_pagestr *)p;
               int value = pg->value;
               if (pg->relative == '+') value += curpage->number;
-              if (value < curpage->number) 
+              if (value < curpage->number)
                 error_moan(112, value, curpage->number, bar, stave);
-              else 
+              else
                 curpage->number = value;
-              }       
-            break; 
+              }
+            break;
 
             case b_pagebots:
             page_sys_botmargin = ((b_pagebotsstr *)p)->value;
@@ -1325,15 +1342,15 @@ otherwise include only those that go inside the stave (staccato, ring, bar).
 This call is used when printing those that go outside the stave.
 
 Arguments:
-  below       TRUE for the bottom bound, FALSE for the top bound 
+  below       TRUE for the bottom bound, FALSE for the top bound
   tie         a tiestr for the note, or NULL if not tied
   accflag     TRUE if there's an accidental
   dynflag     TRUE if there are dynamics
-  
-Returns:      the y value    
+
+Returns:      the y value
 */
 
-int 
+int
 misc_ybound(BOOL below, b_tiestr *tie, BOOL accflag, BOOL dynflag)
 {
 int yield;
@@ -1438,12 +1455,12 @@ relevant. */
 
 if ((bar_cont->flags & cf_bowingabove) == 0)
   {
-  if (!below) acflags &= ~(af_up | af_down); 
+  if (!below) acflags &= ~(af_up | af_down);
   }
 else
   {
-  if (below) acflags &= ~(af_up | af_down); 
-  }     
+  if (below) acflags &= ~(af_up | af_down);
+  }
 
 
 if ((acflags & af_dynamics) != 0 && (dynflag || (acflags & af_dyninside) != 0))
