@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: January 2019 */
 
 
 /* This file contains code for outputting a page */
@@ -286,7 +286,7 @@ int f = p->font;
 int i = 0;
 int *matrix = p->matrix;
 
-uschar *s = p->a.text;
+uschar *s = p->text;
 uschar left[256];
 uschar centre[256];
 uschar right[256];
@@ -363,7 +363,7 @@ while (p != NULL)
   if (p->size > 0)
     {
     int descender = (4 * p->size)/10; 
-    out_yposition += p->b.spaceabove;
+    out_yposition += p->spaceabove;
     if (out_yposition + descender > out_bbox[1]) 
       out_bbox[1] = out_yposition + descender;
     if (out_yposition - p->size < out_bbox[3]) 
@@ -381,7 +381,7 @@ while (p != NULL)
     draw_ox = draw_oy = 0;
     out_stave = -1;
     out_ystave = out_yposition;
-    out_dodraw(p->a.drawing, p->b.args, FALSE);
+    out_dodraw(p->drawing, p->drawargs, FALSE);
     out_yposition += p->space;
     }
 
