@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: January 2019 */
 
 
 /* This file contains code for splitting the music bars into systems and making
@@ -1188,7 +1188,7 @@ while (!page_done) switch(page_state)
       if (page_footnotes == NULL) page_footnotes = page_newfootnotes; else
         {
         page_lastfootnote->next = page_newfootnotes;
-        page_newfootnotes->b.spaceabove = curmovt->footnotesep;
+        page_newfootnotes->spaceabove = curmovt->footnotesep;
         page_footnotedepth += curmovt->footnotesep;
         }
       page_lastfootnote = page_lastnewfootnote;
