@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: January 2019 */
 
 /* This is the main header file, imported by all other sources. */
 
@@ -45,9 +45,12 @@
 #define MAX_DRAW_VARIABLE    19   /* Max number of a draw variable */
 #define MAX_FONTS            50   /* Maximum number of different fonts */
 #define MAX_INCLUDE          10   /* Max depth of included files */
+#define MAX_MACRONAME       256   /* Max length of macro name */
+#define MAX_MACRODEFAULT    256   /* Max length of macro default argument */
+#define MAX_MACROARGS        20   /* Max number of macro arguments */
 #define DEF_MAX_MOVEMENTS    50   /* Default max movements */
 #define MAX_POSTABLESIZE    400   /* Max number of entries */
-#define READ_BUFFERSIZE    1000   /* Input buffer size */
+#define READ_BUFFERSIZE    2000   /* Input buffer size */
 #define MIDI_MAXCHANNEL      16   /* Maximum MIDI channel number */
 #define TEXT_QUEUE_SIZE      50   /* Max texts before a note */
 
@@ -230,23 +233,23 @@ typedef struct tree_node
   }
 tree_node;
 
-/* One heading/footing line. */
+/* One heading/footing/footnote. There are three different cases: a text
+string, a drawing, or a PostScript string. They use different subsets of the
+fields in this structure. */
 
 typedef struct headstr {
   struct headstr *next;
-  int  stretch;          /* amount to stretch spaces by */
-  int  space;            /* amount of space after line */
-  int  size;             /* 0 => postscript heading; < 0 => drawing */
-  int *matrix;           /* points to transformation or NULL */
-  union {
-    uschar *text;        /* either a pointer to a string */
-    tree_node *drawing;  /* or a pointer to a drawing */
-  } a;
-  union {
-    int  spaceabove;     /* either space above a line of text */
-    drawitem *args;      /* or args for a drawing */
-  } b;
-  int  font;             /* initial font */
+  int  size;           /* Font size: 0 => PostScript; < 0 => drawing */
+  int  space;          /* Space to follow; must be 0 except for text */
+  uschar *text;        /* String for text or PostScript */
+  /* --- Text string --- */ 
+  int *matrix;         /* points to transformation or NULL */
+  int  font;           /* initial font */
+  int  stretch;        /* amount to stretch spaces by */
+  int  spaceabove;     /* space above the line of text */
+  /* --- Drawing --- */
+  tree_node *drawing;  /* pointer to the drawing */
+  drawitem *drawargs;  /* drawing arguments */
 } headstr;
 
 /* Text associated with a line or slur gap */
@@ -460,7 +463,7 @@ enum { movt_default, movt_newpage, movt_thispage, movt_thisline,
 
 /* Skip options for errors */
 
-enum { skip_EOL, skip_KET, skip_BACKSLASH };
+enum { skip_OFF, skip_EOL, skip_KET, skip_BACKSLASH, skip_BAR };
 
 /* Notehead type options. If these are changed, the tables in
 shownote.c must also be changed. No stems are printed for values
