@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: January 2019 */
 
 
 /* This file contains code for handling "preprocessing" functions. */
@@ -32,7 +32,7 @@ Returns:   nothing
 void 
 pre_process(void)
 {
-uschar word[256];
+uschar word[MAX_MACRONAME + 1];
 read_word(word);
 sigchNL();
 
@@ -235,6 +235,8 @@ if (Ustrcmp(word, "define") == 0)
       {
       do
         {
+        if (i >= MAX_MACRONAME)  
+          error_moan(136, "name", MAX_MACRONAME);  /* Hard */
         word[i++] = read_ch;
         next_ch();
         }
@@ -247,8 +249,8 @@ if (Ustrcmp(word, "define") == 0)
     {
     int len;
     int argcount = 0; 
-    uschar *args[20]; 
-    uschar arg[256]; 
+    uschar *args[MAX_MACROARGS]; 
+    uschar arg[MAX_MACRODEFAULT + 1]; 
     tree_node *p = store_Xget(sizeof(tree_node));
 
     p->name = store_Xget(Ustrlen(word) + 1);
@@ -260,12 +262,19 @@ if (Ustrcmp(word, "define") == 0)
         {
         int bracount = 0; 
         BOOL inquotes = FALSE; 
-        uschar *s = arg; 
+        uschar *s = arg;
+        
+        if (argcount >= MAX_MACROARGS) 
+          error_moan(137, MAX_MACROARGS);  /* Hard */
+  
         while (++read_chptr < read_endptr && 
               ((*read_chptr != ',' && *read_chptr != ')') ||
                 bracount > 0 || inquotes))  
           {
           int ch = *read_chptr; 
+          if (s - arg >= MAX_MACRODEFAULT) 
+            error_moan(136, "default argument", MAX_MACRODEFAULT);  /* Hard */
+ 
           if (ch == '&') *s++ = *(++read_chptr); else
             { 
             if (ch == '\"') inquotes = !inquotes; 
