@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: January 2019 */
 
 
 /* This file contains code for outputting things in PostScript */
@@ -60,11 +60,11 @@ static int  ps_curfonttransform[6];
 Arguments:
   format      a format string
   ...         arguments for the format
-  
-Returns:      nothing 
+
+Returns:      nothing
 */
 
-static 
+static
 void ps_printf(const char *format, ...)
 {
 int len;
@@ -131,7 +131,7 @@ Arguments:
 Returns:         nothing
 */
 
-static void 
+static void
 ps_setfont(int f, int s, BOOL X)
 {
 ps_curfont = f;
@@ -208,19 +208,19 @@ else
 *               Basic string output code         *
 *************************************************/
 
-/* This function scans a string's characters and converts to the appropriate 
+/* This function scans a string's characters and converts to the appropriate
 output encoding for the PostScript fonts.
 
 Arguments:
   s            the string, in UTF-8 format
   f            the font
   pointsize    the font size
-  absolute     TRUE if the coordinates are absolute, else relative 
+  absolute     TRUE if the coordinates are absolute, else relative
   x            the x coordinate
   y            the y coordinate
   w            extra width for spaces (for justifying)
-  
-Returns:       nothing 
+
+Returns:       nothing
 */
 
 static void
@@ -242,12 +242,12 @@ and pointsize above. */
 
 if (ps_EPS)
   {
-  int descender = (4 * pointsize)/10; 
+  int descender = (4 * pointsize)/10;
   if (y + descender > out_bbox[1]) out_bbox[1] = y + descender;
   if (y - pointsize < out_bbox[3]) out_bbox[3] = y - pointsize;
-  } 
+  }
 
-/* When printing right-to-left, we need to find the complete length of the 
+/* When printing right-to-left, we need to find the complete length of the
 string so that we can print it from the other end, since the fonts still work
 left-to-right. We also need the length for EPS output, in order to set the
 bounding box. Finding the length requires a preliminary scan of the whole
@@ -259,28 +259,28 @@ if (main_righttoleft || ps_EPS)
   int swidth = 0;
   int last_width = 0;
   int last_r2ladjust = 0;
-   
+
   for (;;)
     {
-    int c; 
-    
+    int c;
+
     GETCHARINC(c, p);
-    if (c == 0) break; 
-    
+    if (c == 0) break;
+
     if (c >= 256)
       {
-      if (!stdencoding) 
-        { 
-        last_width = fs->widths[UNKNOWN_CHAR_N]; 
-        last_r2ladjust = fs->r2ladjusts[UNKNOWN_CHAR_N]; 
-        } 
-    
-      else if (c < LOWCHARLIMIT) 
-        { 
-        last_width = fs->widths[c]; 
-        last_r2ladjust = fs->r2ladjusts[c]; 
-        } 
-         
+      if (!stdencoding)
+        {
+        last_width = fs->widths[UNKNOWN_CHAR_N];
+        last_r2ladjust = fs->r2ladjusts[UNKNOWN_CHAR_N];
+        }
+
+      else if (c < LOWCHARLIMIT)
+        {
+        last_width = fs->widths[c];
+        last_r2ladjust = fs->r2ladjusts[c];
+        }
+
       else
         {
         uschar utf[8];
@@ -288,27 +288,27 @@ if (main_righttoleft || ps_EPS)
         utf[misc_ord2utf8(c, utf)] = 0;
         t = Tree_Search(fs->high_tree, utf);
         if (t == NULL)
-          { 
-          last_width = fs->widths[UNKNOWN_CHAR_S]; 
-          last_r2ladjust = fs->r2ladjusts[UNKNOWN_CHAR_S]; 
-          } 
-        else 
           {
-          last_width = t->val[1]; 
-          last_r2ladjust = t->val[2]; 
-          } 
+          last_width = fs->widths[UNKNOWN_CHAR_S];
+          last_r2ladjust = fs->r2ladjusts[UNKNOWN_CHAR_S];
+          }
+        else
+          {
+          last_width = t->val[1];
+          last_r2ladjust = t->val[2];
+          }
         }
       }
-      
+
     else
       {
-      int cj = c; 
-    
-      /* Fudges for various special cases in the music font, where the 
+      int cj = c;
+
+      /* Fudges for various special cases in the music font, where the
       adjustment bounding box is taken from another character. */
-    
+
       if (f == font_mf)
-        { 
+        {
         switch(c)
           {
           case 'J':       /* Additional stem characters - adjust as for note */
@@ -316,49 +316,49 @@ if (main_righttoleft || ps_EPS)
           case 'o':
           case 'p':
           case 'q':
-          case 'r':      
+          case 'r':
           cj = '5';
           break;
-          
+
           case '7':       /* Up quaver - adjust as for down quaver */
           cj = '8';
           break;
-           
+
           case '9':       /* Up semiquaver - adjust as for down semiquaver */
           cj = ':';
           break;
-          
+
           default:
           break;
-          }        
+          }
         }
-   
+
       last_width = fs->widths[c];
       last_r2ladjust = fs->r2ladjusts[cj];
-      } 
-      
+      }
+
     /* Amass the total string width */
-    
+
     swidth += last_width;
-     
+
     /* If there is another character, scan the kerning table */
-  
+
     if (main_kerning && fs->kerncount > 0 && *p != 0)
       {
       int bot = 0;
       int top = fs->kerncount;
       int cc;
       unsigned int pair;
-  
+
       GETCHAR(cc, p);
       pair = (c << 16) | cc;
-  
+
       while (bot < top)
         {
         kerntablestr *k;
         int mid = (bot + top)/2;
         k = &(ktable[mid]);
-  
+
         if (pair == k->pair)
           {
           swidth += k->kwidth;
@@ -367,58 +367,58 @@ if (main_righttoleft || ps_EPS)
         if (pair < k->pair) top = mid; else bot = mid + 1;
         }
       }
-    }  
-    
+    }
+
   /* For right-to-left, adjust the printing position for the string by the
   length of the string, adjusted for the actual bounding box of the final
-  character, and scaled to the font size. For EPS output, adjust the bounding 
+  character, and scaled to the font size. For EPS output, adjust the bounding
   box. Both may, of course, happen. */
 
   if (main_righttoleft)
     x += mac_muldiv(swidth - last_width + last_r2ladjust, pointsize, 1000);
-     
+
   if (ps_EPS)
     {
-    swidth = mac_muldiv(swidth, pointsize, 1000);    
+    swidth = mac_muldiv(swidth, pointsize, 1000);
     if (x + swidth > out_bbox[2]) out_bbox[2] = x + swidth;
-    } 
-  } 
+    }
+  }
 
 /* Scan the string and generate the output. */
 
 p = s;
 for (;;)
   {
-  int c, pc; 
+  int c, pc;
   BOOL extended = FALSE;
 
   GETCHARINC(c, p);
   if (c == 0) break;
-  
-  pc = c;    /* The character value to print */ 
-  
+
+  pc = c;    /* The character value to print */
+
   /* For standardly encoded fonts, code points >= 256 and < LOWCHARLIMIT are
   encoded in the second of the two PostScript fonts, using the Unicode encoding
   less 256. The remaining code points have to be converted to some of the
   remaining characters in the PostScript font, which are encoded arbitrarily,
-  i.e. not using the Unicode encoding (some of their Unicode values are quite 
+  i.e. not using the Unicode encoding (some of their Unicode values are quite
   large). To find this encoding, we search for the character in the widths tree
   for the font, where the offset is also stored. */
-  
+
   if (c >= 256)
     {
-    if (!stdencoding) 
-      { 
+    if (!stdencoding)
+      {
       pc = UNKNOWN_CHAR_N;  /* Known to be < 256 */
-      error_moan(78, c, UNKNOWN_CHAR_N); 
-      } 
+      error_moan(78, c, UNKNOWN_CHAR_N);
+      }
+
+    else if (c < LOWCHARLIMIT)
+      {
+      pc = c - 256;
+      extended = TRUE;
+      }
 
-    else if (c < LOWCHARLIMIT) 
-      { 
-      pc = c - 256; 
-      extended = TRUE; 
-      } 
-       
     else
       {
       uschar utf[8];
@@ -426,18 +426,18 @@ for (;;)
       utf[misc_ord2utf8(c, utf)] = 0;
       t = Tree_Search(fs->high_tree, utf);
       if (t == NULL)
-        { 
+        {
         pc = UNKNOWN_CHAR_S;
-        error_moan(79, c, UNKNOWN_CHAR_S); 
-        } 
-      else 
+        error_moan(79, c, UNKNOWN_CHAR_S);
+        }
+      else
         {
         pc = LOWCHARLIMIT + t->val[0] - 256;
         extended = TRUE;
-        } 
+        }
       }
     }
-    
+
   /* Change between base and extended font if necessary */
 
   if (ps_needchangefont(f, pointsize, extended))
@@ -445,12 +445,12 @@ for (;;)
     if (instring)
       {
       ps_endstring(absolute, w, x, y);
-      x = y = 0; 
+      x = y = 0;
       absolute = instring = FALSE;
       }
     ps_setfont(f, pointsize, extended);
     }
-    
+
   /* Arrange to print the byte */
 
   if (!instring)
@@ -461,17 +461,17 @@ for (;;)
       ps_chcount = 0;
       }
     fputc('(', ps_file);
-    ps_chcount++; 
+    ps_chcount++;
     instring = TRUE;
     }
 
-  if (pc == '(' || pc == ')' || pc == '\\') 
+  if (pc == '(' || pc == ')' || pc == '\\')
     ps_chcount += fprintf(ps_file, "\\%c", pc);
-  else if (pc >= 32 && pc <= 126) 
+  else if (pc >= 32 && pc <= 126)
     {
     fputc(c, ps_file);
     ps_chcount++;
-    }  
+    }
   else ps_chcount += fprintf(ps_file, "\\%03o", pc);
 
   /* If there is another character, scan the kerning table */
@@ -513,7 +513,7 @@ for (;;)
       xadjust = mac_muldiv(xadjust, font_transform[0], 65536);
       ps_endstring(absolute, w, x, y);
       absolute = FALSE;
-      instring = FALSE; 
+      instring = FALSE;
       x = main_righttoleft? -xadjust : xadjust;
       y = yadjust;
       }
@@ -538,11 +538,11 @@ Arguments:
   x          the absolute x coordinate
   y          the absolute y coordinate
   w          the extra space for each space
-  
-Returns:     nothing
-*/      
 
-void 
+Returns:     nothing
+*/
+
+void
 ps_wtext(uschar *s, int font, int pointsize, int x, int y, int w)
 {
 ps_basic_string(s, font, pointsize, TRUE, x, y, w);
@@ -554,8 +554,8 @@ ps_basic_string(s, font, pointsize, TRUE, x, y, w);
 *          Output string in music font           *
 *************************************************/
 
-/* There are two versions, one with absolute coordinates, and one with relative 
-coordinates, to save having to pass a flag each time (most of the calls are 
+/* There are two versions, one with absolute coordinates, and one with relative
+coordinates, to save having to pass a flag each time (most of the calls are
 with absolute coordinates).
 
 Arguments:
@@ -563,17 +563,17 @@ Arguments:
   pointsize  the pointsize for the font
   x          the absolute x coordinate
   y          the absolute y coordinate
-  
-Returns:     nothing
-*/      
 
-void 
+Returns:     nothing
+*/
+
+void
 ps_musstring(uschar *s, int pointsize, int x, int y)
 {
 ps_basic_string(s, font_mf, pointsize, TRUE, x, y, 0);
 }
 
-void 
+void
 ps_relmusstring(uschar *s, int pointsize, int x, int y)
 {
 ps_basic_string(s, font_mf, pointsize, FALSE, x, y, 0);
@@ -597,9 +597,9 @@ Arguments:
   x          pointer to the absolute x coordinate
   y          pointer to the absolute y coordinate
   update     if TRUE, update the x,y positions
-  
+
 Returns:     nothing
-*/      
+*/
 
 void
 ps_string(uschar *s, int f, int pointsize, int *x, int *y, BOOL update)
@@ -618,8 +618,8 @@ if (f == font_mu || f == font_mf)
     {
     int ch;
     GETCHARINC(ch, ss);
-    if (ch < 118 || (ch > 126 && ch < 185) || ch > 188) 
-      { skip = FALSE; /* break; */ } 
+    if (ch < 118 || (ch > 126 && ch < 185) || ch > 188)
+      { skip = FALSE; /* break; */ }
     }
   }
 
@@ -648,22 +648,22 @@ Arguments:
   ytop       the top of the barline
   ybot       the bottom of the barline
   type       the type of barline
-  
-Returns:     nothing
-*/    
 
-void 
+Returns:     nothing
+*/
+
+void
 ps_barline(int x, int ytop, int ybot, int type)
 {
 int magn = (curmovt->barlinesize > 0)? curmovt->barlinesize : main_stavemagn;
 
 /* Normally, solid barlines and dashed ones of a single stave's depth are
 done with characters from the music font, except when the barline size is
-greater than the stave magnification and it's just one stave deep. However, the 
+greater than the stave magnification and it's just one stave deep. However, the
 bar_use_draw option forces all bar lines to be drawn. */
 
 if (!bar_use_draw &&
-    (type != bar_dotted || ytop == ybot) && 
+    (type != bar_dotted || ytop == ybot) &&
     (magn <= main_stavemagn || ytop != ybot))
   {
   if (main_righttoleft)
@@ -683,7 +683,7 @@ magnification - or if bar_use_draw is set. */
 
 else
   {
-  int half_thickness = (type == bar_thick)? magn : 
+  int half_thickness = (type == bar_thick)? magn :
     (type == bar_dotted)? magn/5 : (magn*3)/20;
   int yadjust = main_stavemagn/5;
 
@@ -724,11 +724,11 @@ Arguments:
   ytop       the y coordinate of the top of the brace
   ybot       the y coordinate of the bottom of the brace
   magn       the magnification
-  
-Returns:     nothing
-*/   
 
-void 
+Returns:     nothing
+*/
+
+void
 ps_brace(int x, int ytop, int ybot, int magn)
 {
 ps_printf(" %f %f %f br%s", ((ybot-ytop+16*magn)*23)/12000, psxtran(x)+1500,
@@ -748,11 +748,11 @@ Arguments:
   ytop       the y coordinate of the top of the bracket
   ybot       the y coordinate of the bottom of the bracket
   magn       the magnification
-  
-Returns:     nothing
-*/   
 
-void 
+Returns:     nothing
+*/
+
+void
 ps_bracket(int x, int ytop, int ybot, int magn)
 {
 ps_printf(" %f %f %f k", psxtran(x), psytran(ytop)+16*magn, psytran(ybot));
@@ -765,7 +765,7 @@ ps_printf(" %f %f %f k", psxtran(x), psytran(ytop)+16*magn, psytran(ybot));
 *************************************************/
 
 /* The stavelines parameter will always be > 0. There is now an option to
-draw the stave lines rather than using characters for them (the default). This 
+draw the stave lines rather than using characters for them (the default). This
 helps with screen displays that are using anti-aliasing.
 
 It has been reported that some PostScript interpreters can't handle the
@@ -795,27 +795,27 @@ int x = leftx;
 if (stave_use_draw > 0)
   {
   int gap;
-  int thickness = (stave_use_draw*main_stavemagn)/10; 
-  
+  int thickness = (stave_use_draw*main_stavemagn)/10;
+
   if (ps_chcount > 0) ps_printf("\n");
   switch(stavelines)
     {
     case 1: y -= 4*main_stavemagn;
-    /* Fall through */ 
+    /* Fall through */
     case 2: y -= 4*main_stavemagn;
-    /* Fall through */ 
+    /* Fall through */
     case 3: gap = 8*main_stavemagn;
     break;
-    
+
     default: gap = 4*main_stavemagn;
     break;
-    }  
-    
-  ps_printf("%f %f %f %f %f %d ST\n", psxtran(x), psytran(y), 
-    rightx - leftx, thickness, gap, stavelines);  
-  ps_chcount = 0; 
+    }
+
+  ps_printf("%f %f %f %f %f %d ST\n", psxtran(x), psytran(y),
+    rightx - leftx, thickness, gap, stavelines);
+  ps_chcount = 0;
   return;
-  } 
+  }
 
 /* Output the stave using music font characters */
 
@@ -859,7 +859,7 @@ ps_printf("%s%f %f s", buff, psxtran(main_righttoleft? x:leftx), psytran(y));
 /* If there's a fraction of 10 points left, deal with it */
 
 if (x < rightx)
-  ps_printf(" (%s)%f %f s", sbuff, 
+  ps_printf(" (%s)%f %f s", sbuff,
     psxtran(main_righttoleft? rightx : (rightx - chwidth)), psytran(y));
 
 ps_printf("\n");
@@ -883,11 +883,11 @@ Arguments:
   y          the y coordinate
   ch         the character's identity number
   pointsize  the point size
-  
-Returns:     nothing    
+
+Returns:     nothing
 */
 
-void 
+void
 ps_muschar(int x, int y, int ch, int pointsize)
 {
 uschar s[10];
@@ -900,23 +900,23 @@ for (p = out_mftable_ps[ch]; p != NULL; p = p->next)
   {
   unsigned int c = p->ch;
   int i = 0;
-  int nxfudge = 0;  
-  
+  int nxfudge = 0;
+
   /* Nasty fudge for bracketed accidentals in right-to-left mode: when the
-  brackets come as individual values, swap them round and fudge the spacing of 
+  brackets come as individual values, swap them round and fudge the spacing of
   the remaining chars. This is needed for flats, in practice. */
-  
+
   if (main_righttoleft) switch (c)
     {
     case 139: c = 140; nxfudge = -1600; break;
-    case 140: c = 139; break;  
+    case 140: c = 139; break;
     case 141: c = 142; nxfudge = -1600; break;
     case 142: c = 141; break;
-    default: break;  
-    }   
+    default: break;
+    }
 
   /* Extract up to 4 music font characters from c */
-   
+
   while (c)
     {
     i += misc_ord2utf8(c & 255, s + i);
@@ -928,7 +928,7 @@ for (p = out_mftable_ps[ch]; p != NULL; p = p->next)
     x + mac_muldiv(p->x, pointsize, 10000) + xfudge,
     y - mac_muldiv(p->y, pointsize, 10000), 0);
 
-  xfudge += nxfudge; 
+  xfudge += nxfudge;
   }
 }
 
@@ -938,20 +938,20 @@ for (p = out_mftable_ps[ch]; p != NULL; p = p->next)
 *            Output a beam line                  *
 *************************************************/
 
-/* This function is called several times for a multi-line beam, with the level 
+/* This function is called several times for a multi-line beam, with the level
 number increasing each time. Information about the slope and other features is
 in beam_* variables.
 
 Arguments:
   x0            starting x coordinate, relative to start of bar
   x1            ending x coordinate, relative to start of bar
-  level         level number 
+  level         level number
   levelchange   set nonzero for accellerando and ritardando beams
-  
-Returns:        nothing
-*/   
 
-void 
+Returns:        nothing
+*/
+
+void
 ps_beam(int x0, int x1, int level, int levelchange)
 {
 int y0, y1;
@@ -991,16 +991,16 @@ if (levelchange != 0)
 x0 += out_barx;
 x1 += out_barx;
 
-/* When printing right-to-left, adjust by one note's printing adjustment. 
+/* When printing right-to-left, adjust by one note's printing adjustment.
 The value can't just be read from the font, as it needs fiddling, so we
 just fudge a fixed value. */
 
 if (main_righttoleft)
   {
-  int adjust = sign * mac_muldiv(n_fontsize/2, main_stavemagn, 1000); 
+  int adjust = sign * mac_muldiv(n_fontsize/2, main_stavemagn, 1000);
   x0 -= adjust;
   x1 -= adjust;
-  } 
+  }
 
 ps_printf(" %f %f %f %f %f m",
   depth, psxtran(x1), psytran(y1), psxtran(x0), psytran(y0));
@@ -1016,20 +1016,20 @@ ps_printf(" %f %f %f %f %f m",
 has resulted in a function called out_slur() that uses more primitive output
 functions, and which could in principle be used for all slurs. However, we
 retain ps_slur for complete, non-dashed, curved slurs for compatibility and to
-keep the size of the PostScript down. 
+keep the size of the PostScript down.
 
 Arguments:
   x0         start x coordinate
   y0         start y coordinate
   x1         end x coordinate
-  y1         end y coordinate   
+  y1         end y coordinate
   flags      slur flags
   co         "centre out" adjustment
-  
-Returns:     nothing   
+
+Returns:     nothing
 */
 
-void 
+void
 ps_slur(int x0, int y0, int x1, int y1, int flags, int co)
 {
 int length = x1 - x0;
@@ -1078,11 +1078,11 @@ Arguments:
   y1          end y coordinate
   thickness   line thickness
   flags       for various kinds of line
-  
-Returns:      nothing
-*/      
 
-void 
+Returns:      nothing
+*/
+
+void
 ps_line(int x0, int y0, int x1, int y1, int thickness, int flags)
 {
 uschar *reset = US"";
@@ -1163,11 +1163,11 @@ Arguments:
   y           vector of y coordinates
   count       number of vector elements
   thickness   line thickness
-  
-Returns:      nothing
-*/    
 
-void 
+Returns:      nothing
+*/
+
+void
 ps_lines(int *x, int *y, int count, int thickness)
 {
 int i;
@@ -1192,11 +1192,11 @@ Arguments:
   y            vector of y coordinates
   c            vector of move/line/curve operators
   thickness    thickness of the lines for stroke; negative for fill
-  
-Returns:       nothing    
+
+Returns:       nothing
 */
 
-void 
+void
 ps_path(int *x, int *y, int *c, int thickness)
 {
 while (*c) switch(*c++)
@@ -1234,7 +1234,7 @@ if (ps_gray != 0) ps_printf(" 0 Sg");
 
 /* This function (similar to the one above) is used for fancy slurs, when the
 coordinate system has been rotated and translated so that its origin is at the
-centre of the slur with the x axis joining the endpoints. The coordinates must 
+centre of the slur with the x axis joining the endpoints. The coordinates must
 therefore not use psxtran/psytran.
 
 Arguments:
@@ -1242,11 +1242,11 @@ Arguments:
   y            vector of y coordinates
   c            vector of move/line/curve operators
   thickness    thickness of the lines for stroke; negative for fill only
-  
-Returns:       nothing    
+
+Returns:       nothing
 */
 
-void 
+void
 ps_abspath(int *x, int *y, int *c, int thickness)
 {
 while (*c) switch(*c++)
@@ -1284,11 +1284,11 @@ Arguments:
   s           the string
   x           the x coordinate
   y           the y coordinate
-  
-Returns:      nothing
-*/   
 
-void 
+Returns:      nothing
+*/
+
+void
 ps_pstext(uschar *s, int x, int y)
 {
 ps_printf(" GS %f %f T\n%%User PostScript\n%s", psxtran(x), psytran(y), s);
@@ -1309,7 +1309,7 @@ Argument:  the gray level
 Returns:   nothing
 */
 
-void 
+void
 ps_setgray(int gray)
 {
 ps_gray = gray;
@@ -1327,11 +1327,11 @@ Arguments:
   dashlength    the dash length
   gaplength     the gap length
   caj           the cap-and-join value
-  
-Returns:        nothing
-*/   
 
-void 
+Returns:        nothing
+*/
+
+void
 ps_setdash(int dashlength, int gaplength, int caj)
 {
 if (dashlength != out_dashlength || gaplength != out_dashgaplength)
@@ -1362,21 +1362,21 @@ if (caj != ps_caj)
 *            Gsave and Grestore                  *
 *************************************************/
 
-/* These functions are called from setslur.c when the coordinate system is 
-translated and rotated for the drawing of a fancy slur. They translate directly 
+/* These functions are called from setslur.c when the coordinate system is
+translated and rotated for the drawing of a fancy slur. They translate directly
 into PostScript shorthand for gsave and grestore.
 
 Arguments:  none
 Returns:    nothing
 */
 
-void 
+void
 ps_gsave(void)
 {
 ps_printf(" GS");
 }
 
-void 
+void
 ps_grestore(void)
 {
 ps_printf(" GR");
@@ -1394,7 +1394,7 @@ Argument:   the amount to rotate, in radians
 Returns:    nothing
 */
 
-void 
+void
 ps_rotate(double r)
 {
 if (r != 0.0) ps_printf(" %f R", (int)((r/(4.0 * atan(1.0)))*180000.0));
@@ -1410,11 +1410,11 @@ if (r != 0.0) ps_printf(" %f R", (int)((r/(4.0 * atan(1.0)))*180000.0));
 Arguments:
   x          x coordinate of the new origin
   y          y coordinate of the new origin
-  
-Returns:     nothing
-*/  
 
-void 
+Returns:     nothing
+*/
+
+void
 ps_translate(int x, int y)
 {
 ps_printf(" %f %f T", psxtran(x), psytran(out_ystave - y));
@@ -1436,7 +1436,7 @@ Arguments:   none
 Returns:     nothing
 */
 
-void 
+void
 ps_newmovt(void)
 {
 if (curmovt->leftmargin < 0)    /* Default (not set by user) */
@@ -1460,9 +1460,9 @@ else ps_xmargin = curmovt->leftmargin;
 Arguments:
   barnumber    the bar number
   stave        the stave
-*/    
+*/
 
-void 
+void
 ps_startbar(int barnumber, int stave)
 {
 if (ps_chcount != 0) ps_chcount = BIGNUMBER/2;
@@ -1478,18 +1478,21 @@ ps_chcount = 0;
 *************************************************/
 
 /* This function is called for the main header file, and also for user-supplied
-head/foot or setup files. Certain lines in the header are included only for EPS
-files. They are flagged in the header file with %EPS. Otherwise, if a line
-starts with %, it is copied only if it starts with %%. Blank lines are omitted.
+head/foot or setup files. If the included file is an EPS file, the insert is
+wrapped in save/restore and the a4, showpage, and copypage commands are
+disabled. Certain lines in the header are included only when we are generating
+an EPS file. They are flagged in the header file with %EPS. Otherwise, if a
+line starts with %, it is copied only if it starts with %%. Blank lines are
+omitted.
 
 Arguments:
   s           the file name
-  relativize  if TRUE, relativize non-absolute path to the current input file 
-  
-Returns:      nothing 
+  relativize  if TRUE, relativize non-absolute path to the current input file
+
+Returns:      nothing
 */
 
-static void 
+static void
 ps_include(uschar *s, BOOL relativize)
 {
 FILE *f;
@@ -1502,13 +1505,31 @@ f = Ufopen(name, "r");
 if (f != NULL)
   {
   uschar buff[256];
+  BOOL line1 = TRUE;
+  BOOL insert_eps = FALSE;
+
   while (Ufgets(buff, 256, f) != NULL)
     {
-    if (ps_EPS && Ustrncmp(buff, "%EPS ", 5) == 0) Ufputs(buff+5, ps_file);
-      else if (buff[0] != '\n' && (buff[0] != '%' || buff[1] == '%'))
-        Ufputs(buff, ps_file);
+    if (line1 && Ustrncmp(buff, "%!PS-Adobe", 10) == 0 &&
+                 Ustrstr(buff, "EPSF-") != NULL)
+      {
+      insert_eps = TRUE;
+      fputs("/epspicsave save def/a4{null pop}def\n", ps_file);
+      fputs("/showpage{initgraphics}def/copypage{null pop}def\n", ps_file);
+      }
+
+    else
+      {
+      if (ps_EPS && Ustrncmp(buff, "%EPS ", 5) == 0) Ufputs(buff+5, ps_file);
+        else if (buff[0] != '\n' && (buff[0] != '%' || buff[1] == '%'))
+          Ufputs(buff, ps_file);
+      }
+
+    line1 = FALSE;
     }
+
   if (buff[Ustrlen(buff)-1] != '\n') fputc('\n', ps_file);
+  if (insert_eps) fputs("epspicsave restore\n", ps_file);
   fclose(f);
   ps_chcount = 0;
   }
@@ -1522,19 +1543,19 @@ else error_moan(4, name, strerror(errno));
 *           Output PostScript head/foot          *
 *************************************************/
 
-/* Output literal PostScript for a heading/footing. If the first character is 
+/* Output literal PostScript for a heading/footing. If the first character is
 '<', the remainder is the name of a file. Otherwise, it is literal PostScript.
 
 Argument:   pointer to a headstr
 Returns:    nothing
 */
 
-void 
+void
 ps_headfoot(headstr *p)
 {
 fprintf(ps_file, "\n%%User PostScript\n");
-if (p->a.text[0] == '<') ps_include(p->a.text + 1, TRUE); else
-  fprintf(ps_file, "%s\n", p->a.text);
+if (p->text[0] == '<') ps_include(p->text + 1, TRUE); else
+  fprintf(ps_file, "%s\n", p->text);
 fprintf(ps_file, "%%End user PostScript\n");
 ps_chcount = 0;
 }
@@ -1546,14 +1567,14 @@ ps_chcount = 0;
 *        Include a Music font in the output      *
 *************************************************/
 
-/* This function is called if either PMW-Music or PMW-Alpha is to be included 
+/* This function is called if either PMW-Music or PMW-Alpha is to be included
 in the output.
 
 Argument:  the name of the font
 Returns:   nothing
 */
 
-static void 
+static void
 include_font(const char *name)
 {
 FILE *f = NULL;
@@ -1575,12 +1596,12 @@ if (f == NULL)
   f = Ufopen(buff, "r");
   if (f == NULL)                     /* These are hard errors */
     {
-    if (ps_fontdir_extra == NULL)  
+    if (ps_fontdir_extra == NULL)
       error_moan(7, buff, strerror(errno));
     else
       error_moan(130, name, ps_fontdir_extra, ps_fontdir_default);
-    }       
-  } 
+    }
+  }
 
 while (Ufgets(buff, sizeof(buff), f) != NULL)
   {
@@ -1607,13 +1628,13 @@ while (Ufgets(buff, sizeof(buff), f) != NULL)
 
 /* This is the controlling function for generating PostScript output. If the
 print_imposition has the special value pc_EPS, we are producing EPS PostScript,
-and a number of page-related parameters are then ignored. 
+and a number of page-related parameters are then ignored.
 
 Arguments: none
 Returns:   nothing
 */
 
-void 
+void
 ps_go(void)
 {
 BOOL used_pmw_music = FALSE;
@@ -1622,7 +1643,7 @@ time_t timer;
 int i, w = 0, d = 0;
 int count = 0;
 int fcount = 1;
-int scaled_main_sheetwidth = 
+int scaled_main_sheetwidth =
   mac_muldiv(main_sheetwidth, print_magnification, 1000);
 
 /* Initialize the current page number and page list data */
@@ -1714,7 +1735,7 @@ if (output_includefont && (used_pmw_music || used_pmw_alpha))
     used_pmw_music? " PMW-Music":"", used_pmw_alpha? " PMW-Alpha":"");
   }
 
-if (!ps_EPS) fprintf(ps_file, 
+if (!ps_EPS) fprintf(ps_file,
   "%%%%Requirements: numcopies(%d)\n", output_copies);
 fprintf(ps_file, "%%%%EndComments\n\n");
 
@@ -1724,34 +1745,34 @@ switch (opt_sheetsize)
   {
   case sheet_A3:
   fprintf(ps_file, "%%%%BeginPaperSize: a3\na3\n%%%%EndPaperSize\n\n");
-  break;  
-  
+  break;
+
   case sheet_A4:
   fprintf(ps_file, "%%%%BeginPaperSize: a4\na4\n%%%%EndPaperSize\n\n");
   break;
-   
+
   case sheet_A5:
   fprintf(ps_file, "%%%%BeginPaperSize: a5\na5\n%%%%EndPaperSize\n\n");
   break;
-  
+
   case sheet_B5:
   fprintf(ps_file, "%%%%BeginPaperSize: b5\nb5\n%%%%EndPaperSize\n\n");
   break;
-  
+
   case sheet_letter:
   fprintf(ps_file, "%%%%BeginPaperSize: letter\nletter\n%%%%EndPaperSize\n\n");
   break;
-  
+
   default:
-  break; 
-  }      
+  break;
+  }
 
 /* Next, the file's prologue */
 
 fprintf(ps_file, "%%%%BeginProlog\n");
 
 /* If there is a header file, copy it now. Its name is NOT relative to the
-main input file. (If it is not absolute, it is taken relative to the current 
+main input file. (If it is not absolute, it is taken relative to the current
 directory.) */
 
 if (ps_header != NULL) ps_include(ps_header, FALSE);
@@ -1764,8 +1785,8 @@ if (main_pssetup != NULL)
   fprintf(ps_file, "\n%% Included pssetup strings and/or files\n");
   while (h != NULL)
     {
-    if (h->a.text[0] == '<') ps_include(h->a.text + 1, TRUE);
-      else fprintf(ps_file, "%s\n", h->a.text);
+    if (h->text[0] == '<') ps_include(h->text + 1, TRUE);
+      else fprintf(ps_file, "%s\n", h->text);
     h = h->next;
     }
   fprintf(ps_file, "\n");
@@ -1868,7 +1889,7 @@ for (;;)
   if (ps_EPS)
     {
     if (main_righttoleft)
-      ps_printf("%f 0 T -1 1 scale\n", main_sheetwidth); 
+      ps_printf("%f 0 T -1 1 scale\n", main_sheetwidth);
     if (main_magnification != 1000)
       ps_printf("%f dup scale\n", main_magnification);
     }
@@ -1879,8 +1900,8 @@ for (;;)
       pagestr *temp = ps_1stpage;
       ps_1stpage = ps_2ndpage;
       ps_2ndpage = temp;
-      }      
-     
+      }
+
     /* Move the origin to the desired position. The values 1 (upright, 1-up,
     portrait), 2 (sideways, 2-up, portrait), and 4 (sideways, 1-up, landscape)
     use bottom left, i.e. no translation, but we have to generate an adjustment
@@ -1919,7 +1940,7 @@ for (;;)
 
       case 5: /* Upright, 1-up, landscape; page size defined by sheetsize */
               /* Sheetwidth is original sheet height */
-      ps_printf("0 %f T -90 R\n", scaled_main_sheetwidth); 
+      ps_printf("0 %f T -90 R\n", scaled_main_sheetwidth);
       break;
 
       case 6: /* A4 Sideways, 2-up, landscape */
@@ -1935,7 +1956,7 @@ for (;;)
       ps_printf("%f %f T\n", print_image_xadjust, print_image_yadjust);
 
     if (main_righttoleft)
-      ps_printf("%f 0 T -1 1 scale\n", scaled_main_sheetwidth); 
+      ps_printf("%f 0 T -1 1 scale\n", scaled_main_sheetwidth);
 
     if (main_magnification != 1000 || print_magnification != 1000)
       {
@@ -1943,7 +1964,7 @@ for (;;)
       ps_printf("%f dup scale\n", scaled);
       }
     }
- 
+
   /* End of setup */
 
   fprintf(ps_file, "%%%%EndPageSetup\n");
@@ -1965,10 +1986,10 @@ for (;;)
     if (ps_chcount > 0) fprintf(ps_file, "\n");
     if (print_imposition == pc_a5ona4 || print_imposition == pc_a4ona3)
       {
-      int sign = main_righttoleft? -1 : +1; 
+      int sign = main_righttoleft? -1 : +1;
       int dd = mac_muldiv(d, 500, scaled);
       if (opt_landscape) ps_printf("0 %f T\n", -dd); else
-        ps_printf("%f 0 T\n", sign * (print_pamphlet? 
+        ps_printf("%f 0 T\n", sign * (print_pamphlet?
           mac_muldiv(main_sheetwidth, 1000, main_magnification) : dd));
       }
     curpage = ps_2ndpage;
@@ -1988,12 +2009,12 @@ fprintf(ps_file, "%%%%Trailer\n");
 
 if (ps_EPS)
   {
-  if (main_righttoleft) 
+  if (main_righttoleft)
     ps_printf("%%%%BoundingBox: %f %f %f %f\n",
-      main_sheetwidth - 
+      main_sheetwidth -
         mac_muldiv(psxtran(out_bbox[2]), main_magnification, 1000),
       mac_muldiv(psytran(out_bbox[1]), main_magnification, 1000),
-      main_sheetwidth - 
+      main_sheetwidth -
         mac_muldiv(psxtran(out_bbox[0]), main_magnification, 1000),
       mac_muldiv(psytran(out_bbox[3]), main_magnification, 1000));
   else
@@ -2002,7 +2023,7 @@ if (ps_EPS)
       mac_muldiv(psytran(out_bbox[1]), main_magnification, 1000),
       mac_muldiv(psxtran(out_bbox[2]), main_magnification, 1000),
       mac_muldiv(psytran(out_bbox[3]), main_magnification, 1000));
-  }   
+  }
 else fprintf(ps_file, "%%%%Pages: %d\n", count);
 }
 
