@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 2nd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: January 2019 */
 
 
 /* This file contains part I of the code for reading in a PMW score file. */
@@ -35,6 +35,10 @@ called to check strings and may need to handle a page number. */
 
 static pagestr dummy_page = { NULL, NULL, NULL, 1, 0, 0, 0 };
 
+/* Default "macro" for the &* replication feature */
+
+static macrostr replicate_macro = { 1, US"&1", { US"" } };
+
 
 
 
@@ -88,137 +92,216 @@ while (inptr < inend)
     break;
     }
 
-  /* Deal with defined insertions, possibly with arguments */
+  /* Deal with defined insertions, possibly with arguments. Also deal with &*()
+  repetitions, which uses very similar code. */
 
   else if (ch == '&')
     {
     if (*inptr == '&') *temp++ = *inptr++; else
       {
-      int i = 0;
-      uschar name[20];
+      macrostr *mm = NULL;
+      BOOL had_semicolon = FALSE; 
+      unsigned int count = 1;
+
+      /* Set up for a macro call */
+      
       if (isalnum(*inptr))
         {
+        int i = 0;
         tree_node *s;
+        uschar name[MAX_MACRONAME+1];
+
         name[i++] = *inptr++;
-        while (isalnum(*inptr)) name[i++] = *inptr++;
+        while (isalnum(*inptr)) 
+          {
+          if (i >= MAX_MACRONAME) 
+            error_moan(136, "name", MAX_MACRONAME);  /* Hard */
+          name[i++] = *inptr++;
+          } 
         name[i] = 0;
+        
+        if (*inptr == ';')
+          {
+          inptr++;                /* Optional semicolon after name is */
+          had_semicolon = TRUE;   /*   skipped, and no args allowed */
+          }    
+ 
         if ((s = Tree_Search(define_tree, name)) != NULL)
           {
-          if (s->data != NULL)
+          mm = (macrostr *)s->data;  /* Will be NULL for no replacement */
+          if (mm == NULL) goto END_MACRO;
+          }
+        else
+          {
+          error_moan(9, name);              /* Couldn't find name */
+          goto END_MACRO;
+          }
+        }
+
+      /* Set up for a replication call */
+
+      else if (*inptr == '*')
+        {
+        int len;
+        if (sscanf((char *)(++inptr), "%u(%n", &count, &len) <= 0)
+          {
+          error_moan(10, "Unsigned number followed by \"(\"");
+          goto END_MACRO;
+          }
+        inptr += len - 1;
+        mm = &replicate_macro;
+        }
+
+      else
+        {
+        error_moan(8);                      /* Bad uschar after '&' */
+        goto END_MACRO;
+        }
+
+      /* Found a macro or &*<digits>. For a macro, mm points to its data, and
+      count is 1. For a replication, mm points to a dummy with 1 empty default
+      argument, count contains the replication count, and we know there is an
+      argument. */
+
+      /* Optimize the case when macro is defined with no arguments */
+
+      if (mm->argcount == 0)
+        {
+        int k = Ustrlen(mm->text);
+        if (outlen - 2 - (temp - outbuffer) < k + (inend - inptr))
+          error_moan(88);   /* hard error - buffer overflow */
+        Ustrcpy(temp, mm->text);
+        temp += k;
+        }
+
+      /* Otherwise we have to process char by char, and read arguments, if
+      any. There need not be; they can all be defaulted. Arguments are read,
+      serially, into argbuff, and then expanded for nested macros into what
+      remains of argbuff. */
+
+      else
+        {
+        int i;
+        int argcount = mm->argcount;
+        uschar *args[MAX_MACROARGS];
+        uschar argbuff[4*READ_BUFFERSIZE];
+        uschar *ap = argbuff;
+
+        /* Set up the default arguments */
+
+        for (i = 0; i < argcount; i++) args[i] = mm->args[i];
+
+        /* Read given arguments, if any, increasing the count if more
+        than the default number. */
+
+        if (!had_semicolon && *inptr == '(')
+          {
+          for (i = 0;; i++)
             {
-            macrostr *mm = (macrostr *)s->data;
-            int k = Ustrlen(mm->text);
-
-            /* Optimize the case with no arguments */
-
-            if (mm->argcount == 0)
+            int bracount = 0;
+            BOOL inquotes = FALSE;
+            uschar *ss = ap;
+            
+            if (argcount >= MAX_MACROARGS) 
+              error_moan(137, MAX_MACROARGS);  /* Hard */
+ 
+            while (++inptr < inend &&
+                  ((*inptr != ',' && *inptr != ')') ||
+                    bracount > 0 || inquotes))
               {
-              if (outlen - 2 - (temp - outbuffer) < k + (inend - inptr))
-                error_moan(88);   /* hard error - buffer overflow */
-              Ustrcpy(temp, mm->text);
-              temp += k;
+              int cch = *inptr;
+              if (cch == '&' && !isalnum(inptr[1]) && inptr[1] != '*')
+                *ap++ = *(++inptr);
+              else
+                {
+                if (cch == '\"') inquotes = !inquotes;
+                if (!inquotes)
+                  {
+                  if (cch == '(') bracount++;
+                    else if (cch == ')') bracount--;
+                  }
+                *ap++ = cch;
+                }
               }
 
-            /* Otherwise have to process char by char, and read arguments, if
-            any. */
+            if (inptr >= inend) error_moan(99);
+             
+            if (i >= argcount) 
+              { 
+              args[i] = NULL; 
+              argcount++; 
+              }
+               
+            if (ap - ss > 0) 
+              { 
+              *ap++ = 0; 
+              args[i] = ss; 
+              }
 
+            if (inptr >= inend || *inptr == ')')
+              { 
+              inptr++; 
+              break; 
+              }
+            }
+
+          /* Only one argument is currently allowed for a replication. Any
+          others are ignored. */
+
+          if (mm == &replicate_macro && argcount > 1) error_moan(134);
+          }
+          
+        /* Process the arguments for nested macro calls */
+
+        for (i = 0; i < argcount; i++)
+          {
+          uschar *new_ap;
+          if (args[i] == NULL || Ustrchr(args[i], '&') == NULL) continue;
+          new_ap = expand_string(args[i], args[i] + Ustrlen(args[i]),
+            ap, argbuff + sizeof(argbuff) - ap, FALSE);
+          args[i] = ap;
+          ap = new_ap + 1;  /* final zero must remain */
+          }
+
+        /* Now copy the replacement, inserting the args. For a replication we
+        repeat many times. For a macro, count is always 1. */
+
+        while (count-- > 0)
+          {
+          uschar *pp = mm->text;
+
+          while (*pp != 0)
+            {
+            if (*pp == '&' && isdigit(pp[1]))
+              {
+              int arg = 0;
+              while (isdigit(*(++pp))) arg = arg*10 + *pp - '0';
+              if (*pp == ';') pp++;
+              if (--arg < argcount)
+                {
+                uschar *ss = args[arg];
+                if (ss != NULL)
+                  {
+                  if (READ_BUFFERSIZE - 2 - (temp - outbuffer) <
+                    Ustrlen(ss)) error_moan(88);  /* hard */
+                  Ustrcpy(temp, ss);
+                  temp += Ustrlen(ss);
+                  }
+                }
+              }
             else
               {
-              int argcount = mm->argcount;
-              uschar *pp = mm->text;
-              uschar *args[20];
-              uschar argbuff[2048];
-              uschar *ap = argbuff;
-
-              /* Set up the default arguments */
-
-              for (i = 0; i < argcount; i++) args[i] = mm->args[i];
-
-              /* Read given arguments, if any, increasing the count if more
-              than the default number. */
-
-              if (*inptr == '(')
-                {
-                for (i = 0;; i++)
-                  {
-                  int bracount = 0;
-                  BOOL inquotes = FALSE;
-                  uschar *ss = ap;
-                  while (++inptr < inend &&
-                        ((*inptr != ',' && *inptr != ')') ||
-                          bracount > 0 || inquotes))
-                    {
-                    int cch = *inptr;
-                    if (cch == '&' && !isalnum(inptr[1]))
-                      *ap++ = *(++inptr);
-                    else
-                      {
-                      if (cch == '\"') inquotes = !inquotes;
-                      if (!inquotes)
-                        {
-                        if (cch == '(') bracount++;
-                          else if (cch == ')') bracount--;
-                        }
-                      *ap++ = cch;
-                      }
-                    }
-
-                  if (inptr >= inend) error_moan(99);
-                  if (i >= argcount) { args[i] = NULL; argcount++; }
-                  if (ap - ss > 0) { *ap++ = 0; args[i] = ss; }
-
-                  if (inptr >= inend || *inptr == ')')
-                    { inptr++; break; }
-                  }
-                }
-
-              /* Process the arguments for nested macro calls */
-
-              for (i = 0; i < argcount; i++)
-                {
-                uschar *new_ap;
-                if (args[i] == NULL || Ustrchr(args[i], '&') == NULL) continue;
-                new_ap = expand_string(args[i], args[i] + Ustrlen(args[i]),
-                  ap, argbuff + sizeof(argbuff) - ap, FALSE);
-                args[i] = ap;
-                ap = new_ap + 1;  /* final zero must remain */
-                }
-
-              /* Now copy the replacement, inserting the args */
-
-              while (*pp != 0)
-                {
-                if (*pp == '&' && isdigit(pp[1]))
-                  {
-                  int arg = 0;
-                  while (isdigit(*(++pp))) arg = arg*10 + *pp - '0';
-                  if (*pp == ';') pp++;
-                  if (--arg < argcount)
-                    {
-                    uschar *ss = args[arg];
-                    if (ss != NULL)
-                      {
-                      if (READ_BUFFERSIZE - 2 - (temp - outbuffer) <
-                        Ustrlen(ss)) error_moan(88);  /* hard */
-                      Ustrcpy(temp, ss);
-                      temp += Ustrlen(ss);
-                      }
-                    }
-                  }
-                else
-                  {
-                  if (temp - outbuffer + 2 >= outlen)
-                    error_moan(88);    /* hard error - buffer overflow */
-                  *temp++ = *pp++;
-                  }
-                }
+              if (temp - outbuffer + 2 >= outlen)
+                error_moan(88);    /* hard error - buffer overflow */
+              *temp++ = *pp++;
               }
             }
           }
-        else error_moan(9, name);              /* Couldn't find name */
-        if (*inptr == ';') inptr++;
         }
-      else error_moan(8);                      /* Bad uschar after '&' */
+
+      /* Macro/replication processing is done */
+
+      END_MACRO: continue;
       }
     }
 
@@ -278,18 +361,48 @@ for (;;)   /* Loop until a character is obtained or very end is reached */
     this_buffer = temp;
     }
 
-  /* Get next line; at end of file, check for missing "fi"s and deal with
-  included files. */
+  /* Get next logical line, joining together physical lines that end with &&&.
+  At end of file, check for missing "fi"s and deal with included files. */
 
-  for (;;)
+  for (;;)  /* Loop for included files */
     {
     if (input_file != NULL)
       {
-      if (Ufgets(this_buffer, READ_BUFFERSIZE - 4, input_file) != NULL) break;
+      BOOL line_read = FALSE;
+      uschar *tbuffer = this_buffer;
+      int size = READ_BUFFERSIZE - 4;
+
+      len = 0;
+      for (;;)
+        {
+        if (Ufgets(tbuffer, size, input_file) != NULL)
+          {
+          int tlen = Ustrlen(tbuffer);
+          line_read = TRUE;
+          read_linenumber++;
+          len += tlen;
+          if (tlen < 4 || Ustrcmp(tbuffer + tlen - 4, "&&&\n") != 0)
+            goto PROCESS_LINE;
+          len -= 4;
+          tlen -= 4;
+          tbuffer += tlen;
+          *tbuffer = 0;
+          size -= tlen;
+          }
+        else  /* Reached EOF */
+          {
+          if (line_read) goto PROCESS_LINE; else break;
+          }
+        }
+
+      /* No lines read */
+
       fclose(input_file);
       input_file = NULL;
       }
 
+    /* Handle reaching the end of an input file */
+
     Ustrcpy(this_buffer, "--- End of file ---");  /* for reflection */
     read_chptr = this_buffer + 19;                /* just in case */
     read_endptr = read_chptr + 1;                 /* nothing left */
@@ -317,11 +430,9 @@ for (;;)   /* Loop until a character is obtained or very end is reached */
     }
 
   /* Another line has been read; take care with the final one which may not
-  have a newline on the end. */
-
-  read_linenumber++;
-  len = Ustrlen(this_buffer);
+  have a newline on the end. Nor will a final line that is just "&&&\n". */
 
+  PROCESS_LINE:
   read_count += len;
   read_chptr  = this_buffer;
   read_endptr = this_buffer + len;
@@ -569,7 +680,7 @@ return key;
 
 /* The global variables main_notenum and main_noteden contain the numerator and
 denominator of any note scaling (doublenotes, halvenotes) that was set up in
-the movement's header. This affects time signatures. The stave directives 
+the movement's header. This affects time signatures. The stave directives
 [doublenotes] and [halvenotes] do not affect time signatures.
 
 Argument:   the time signature
@@ -965,7 +1076,7 @@ if (main_rc < rc_failed) for (;;)
       stave_notes = TRUE;
       stave_noteflags = (curstave == 0)? nf_hidden : 0;
       stave_notenum = main_notenum;
-      stave_noteden = main_noteden;  
+      stave_noteden = main_noteden;
       stave_matchnum = 0; /* indicates "unset" */
       stave_totalnocount = stave_slurcount =
         stave_hairpinbegun = stave_hairpiny = 0;
