@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: January 2019 */
 
 
 /* This file contains part II of the code for reading in a PMW score file. It
@@ -405,16 +405,18 @@ void
 read_headfootingtext(headstr *new, int type)
 {
 int defaultsize = 0;
+BOOL is_pshdr = (type & rh_ps) != 0;
 
 new->next = NULL;
 new->stretch = 0;
 new->font = font_rm;
 
-/* If the next character is a letter, we expect to read "draw <name>";
-otherwise we expect an optional type size and a text string. */
+/* For a non-PostScript header, if the next character is a letter, we expect to
+read "draw <name>"; otherwise we expect an optional type size and a text
+string. */
 
 sigch();
-if (isalpha(read_ch))
+if (!is_pshdr && isalpha(read_ch))
   {
   tree_node *node;
   int argcount = 0;
@@ -461,20 +463,21 @@ if (isalpha(read_ch))
     error_moan(70, word);
     return;
     }
-  new->a.drawing = node;
-  new->b.args = drawargs;
+  new->drawing = node;
+  new->drawargs = drawargs;
   new->size = -1;
   }
 
-/* Deal with conventional heading */
+/* Deal with PostScript and non-draw heading/footing */
 
 else
   {
-  /* Set the type size, either by reading, or by algorithm. If explicit,
-  can be followed by stretch and possibly shear, otherwise a NULL matrix. */
+  /* In the non-PostScript case, set the type size, either by reading, or by
+  algorithm. If explicit, can be followed by stretch and possibly shear,
+  otherwise a NULL matrix. */
 
   new->matrix = NULL;
-  if (isdigit(read_ch))
+  if (!is_pshdr && isdigit(read_ch))
     {
     new->size = read_integer(TRUE);
     if (read_ch == '/')
@@ -518,8 +521,8 @@ else
     new->matrix = curmovt->fontsizes->fontmatrix_footnote;
     break;
 
-    default:    /* PostScript headings/footings */
-    new->size = 0;
+    default:        /* PostScript headings/footings */
+    new->size = 0;  /* This indicates that it's PostScript */
     break;
     }
 
@@ -534,23 +537,26 @@ else
 
   /* We now expect a heading string; if not PostScript, check its escapes */
 
-  new->a.text = string_read();
-  new->b.spaceabove = 0;
-  if (new->a.text != NULL && new->size != 0)
-    new->a.text = string_check(new->a.text);
+  new->text = string_read();
+  new->spaceabove = 0;
+  if (new->text != NULL && !is_pshdr) new->text = string_check(new->text);
   }
 
 /* The stretch value is set to zero here; if required it will be set later. */
 
 new->stretch = 0;
 
-/* If another number follows, it is the space which follows; if not, default
-it. */
+/* For a non-PostScript header, if another number follows, it is the space that
+follows the heading; if not, default it. */
 
 sigch();
-if (isdigit(read_ch) || read_ch == '+' || read_ch == '-')
-  read_expect_integer(&(new->space), TRUE, TRUE);
-else new->space = defaultsize;
+if (!is_pshdr)
+  {
+  if (isdigit(read_ch) || ((read_ch == '+' || read_ch == '-') && 
+      isdigit(*read_chptr)))
+    read_expect_integer(&(new->space), TRUE, TRUE);
+  else new->space = defaultsize;
+  } 
 }
 
 
@@ -1559,9 +1565,9 @@ if (main_lastmovement != 1) error_moan(20); else
     {
     h = store_Xget(sizeof(headstr));
     h->next = NULL;
-    h->size = -1;
-    h->a.text = store_copystring(s);
-    h->b.spaceabove = 0;
+    h->size = 0;
+    h->text = store_copystring(s);
+    h->spaceabove = 0;
     *hh = h;
     }
 
