@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: January 2019 */
 
 
 /* This file contains part III of the code for reading in a PMW score file. It
@@ -313,15 +313,22 @@ for (;;)
             {
             int x;
             next_ch();
+            error_skip = skip_OFF;  /* Temporarily no skip */ 
             if (read_expect_integer(&x, FALSE, FALSE))
               {
               if (x <= dyn_max)
                 stave_accentflags |= accent_list[x];
               else error_moan(128, x);
               }
+            error_skip = skip_KET;   
             }
 
-          else { error_moan(45); next_sigch(); }
+          else 
+            { 
+            error_moan(45);
+            if (read_ch == ']') break;
+            next_sigch(); 
+            }
           }
 
         if ((stave_accentflags & (af_staccato|af_staccatiss)) ==
@@ -767,6 +774,7 @@ for (;;)
       {
       (void)store_getitem(b_rrepeat);
       next_ch();
+      if (stave_barlength == 0) error_moan(138, "End", "start");  /* Warning */ 
       }
     else (void)store_getitem(b_dotbar);
     break;
@@ -799,7 +807,8 @@ for (;;)
       {
       store_getitem(b_lrepeat);
       next_ch();
-      next_ch();
+      next_sigch();
+      if (read_ch == '|') error_moan(138, "Start", "end");  /* Warning */
       break;
       }
 
