@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: January 2019 */
 
 
 /* This file contains part IV of the code for reading in a PMW score file. It
@@ -2319,6 +2319,7 @@ static dirstr read_stavedirlist[] = {
   { "rmove",         p_move,       TRUE, FALSE },
   { "rsmove",        p_smove,      TRUE, FALSE },
   { "rspace",        p_rspace,     TRUE, FALSE },
+  { "sgabove",       p_sg,         b_sgabove, TRUE },
   { "sghere",        p_sg,         b_sghere, TRUE },
   { "sgnext",        p_sg,         b_sgnext, TRUE },
   { "skip",          p_skip,       0, TRUE },
