@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: January 2019 */
 
 
 /* This file contains part V of the code for reading in a PMW score file - code
@@ -445,9 +445,12 @@ else if (length == 0)
   stave_lastgracestem = stemforce;
   }
 
-/* Unset grace stem forcing on a rest */
+/* For a rest, unset grace stem forcing */
 
-else stave_lastgracestem = 0;
+else 
+  {
+  stave_lastgracestem = 0;
+  } 
 
 /* We now need to deal with ties, glissando marks, beam breaks, and the ends of
 plet groups. We permit the plet group ending to come before any of the other
@@ -545,7 +548,7 @@ if (opt_oldbeambreak)
       if (stave_beaming) read_setbeamstems();
       }
     }
-  else if (read_ch == ',') goto SECBEAMBREAK;   
+  else if (read_ch == ',') goto SECBEAMBREAK;
   }
 
 else  /* Not oldbeambreak */
@@ -562,7 +565,7 @@ else  /* Not oldbeambreak */
   else if (read_ch == ',')
     {
     unsigned int v;
-    SECBEAMBREAK: 
+    SECBEAMBREAK:
     next_ch();
     if (isdigit(read_ch))
       {
@@ -576,7 +579,7 @@ else  /* Not oldbeambreak */
       b->value = v;
       }
     }
-  }   
+  }
 
 /* If we encountered '}' previously or here, end plet group */
 
@@ -668,6 +671,16 @@ if (read_ch == 'x')
   int count = 1;
   next_sigch();
   if (isdigit(read_ch)) count = read_integer(FALSE);
+  
+  if (read_ch == '\\')  /* No options are allowed */
+    {
+    next_ch(); 
+    error_skip = skip_BACKSLASH;
+    error_moan(135);
+    error_skip = skip_EOL;  
+    next_ch(); 
+    }  
+ 
   if (stave_lastbasenoteptr == NULL) { error_moan(115); return; }
 
   for (; count > 0; count--)
@@ -677,7 +690,7 @@ if (read_ch == 'x')
     b_notestr *new;
     bstr *p = (bstr *)last_note_ornament;
     do_underlay();
-    chordcount = 0; 
+    chordcount = 0;
 
     if (p != NULL) while (p != (bstr *)old)
       {
@@ -718,10 +731,10 @@ if (read_ch == 'x')
     stave_pitchtotal += new->truepitch;
     stave_pitchcount++;
     chordcount++;
-    
+
     /* This error should never occur because the original chord should be
-    diagnosed. Paranoia. */ 
- 
+    diagnosed. Paranoia. */
+
     if (chordcount >= MAX_CHORDSIZE) error_moan(132, MAX_CHORDSIZE); /* Hard */
     if (new->spitch > maxpitch) maxpitch = new->spitch;
     if (new->spitch < minpitch) minpitch = new->spitch;
@@ -1003,9 +1016,11 @@ for (;;)
       else if (read_ch == 's') nopack = TRUE;
         else if (read_ch != 'r')
           {
-          error_moan(37, inchord? "Note letter or end of chord" : "Note or rest letter");
-          pitch = 30;               /* Take a random pitch */
-          inchord = FALSE;          /* Terminate a chord, in case missing ')' */
+          error_moan(37, inchord? "Note letter or end of chord" : 
+                                  "Note or rest letter");
+          pitch = 30;                 /* Take a random pitch */
+          inchord = FALSE;            /* Terminate a chord, in case missing ')' */
+          stave_checklength = FALSE;  /* Disable length check for this bar */
           }
     if (inchord && pitch == 0) error_moan(43);
     }
@@ -1268,9 +1283,30 @@ TRANSPOSE:
 
   if (read_ch == '\\')
     {
+    BOOL had_error = FALSE;
+    uschar *end_options;
+
     error_skip = skip_BACKSLASH;
     next_sigch();
 
+    /* Search for the terminating backslash in the current line, to try to
+    minimise error cascades when it is missing. */
+
+    for (end_options = read_chptr;; end_options++)
+      { 
+      if (end_options >= read_endptr ||
+          (*end_options == '|' && end_options[-1] != '~' &&
+            end_options[-1] != 't'))
+        {
+        error_skip = skip_BAR;
+        error_moan(133);
+        goto ENDACCENT;
+        }
+      if (*end_options == '\\') break;
+      } 
+
+    /* Now process the options */
+
     while (read_ch != '\\' && read_ch != EOF)
       {
       accent *ap;
@@ -1531,6 +1567,7 @@ TRANSPOSE:
           {
           int x;
           next_ch();
+          error_skip = skip_OFF;  /* Temporarily no skip */
           if (read_expect_integer(&x, FALSE, FALSE))
             {
             if (chordcount == 0)
@@ -1544,11 +1581,27 @@ TRANSPOSE:
               }
             else error_moan(48, "Accents");
             }
+          error_skip = skip_BACKSLASH;
           }
 
-        /* Unknown */
+        /* Unknown ornament or expression. Give up after two successive errors. 
+        */
 
-        else { error_moan(45); next_sigch(); }
+        else
+          {
+          error_moan(45);
+          if (had_error)
+            {
+            fprintf(stderr, "** Assuming missing backslash after two "
+              "expression/ornament errors\n\n");
+            goto ENDACCENT;
+            }
+          else
+            {
+            had_error = TRUE;
+            next_sigch();
+            }
+          }
         }
 
       if ((acflags & (af_staccato|af_staccatiss)) ==
@@ -1560,6 +1613,7 @@ TRANSPOSE:
     }
 
   ENDACCENT:                       /* some error states jump here */
+  error_skip = skip_EOL;           /* default setting */
 
   /* Rests should not have ornaments or accidentals */
 
