@@ -2,10 +2,10 @@
 *    The PMS Music Typesetter - 2nd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: January 2019 */
 
 
 /* This file contains code for reading and processing strings. */
@@ -636,13 +636,15 @@ if (*s == 'x' && isdigit(s[1]))
 
 *revert = TRUE;
 
-/* Deal with ligatures */
+/* Deal with ligatures and double typographic quotes */
 
 if      (Ustrncmp(s, "fi",  2) == 0) { unicode = 0xfb01; s += 2; }
 else if (Ustrncmp(s, "fl",  2) == 0) { unicode = 0xfb02; s += 2; }
 else if (Ustrncmp(s, "ss",  2) == 0) { unicode = 0x00df; s += 2; }
 else if (Ustrncmp(s, "---", 3) == 0) { unicode = 0x2014; s += 3; }
 else if (Ustrncmp(s, "--",  2) == 0) { unicode = 0x2013; s += 2; }
+else if (Ustrncmp(s, "<<",  2) == 0) { unicode = 0x201C; s += 2; }
+else if (Ustrncmp(s, ">>",  2) == 0) { unicode = 0x201D; s += 2; }
 
 else switch (*s)
   {
