@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: January 2019 */
 
 
 /* This file contains all the major tables used by the program; a few that are
@@ -819,6 +819,7 @@ uschar length_table [] = {
   ROUND(sizeof(b_settimestr)),
   ROUND(sizeof(b_sgstr)),
   ROUND(sizeof(b_sgstr)),
+  ROUND(sizeof(b_sgstr)),
   ROUND(sizeof(b_slopestr)),
   ROUND(sizeof(b_slurstr)),
   ROUND(sizeof(b_linegapstr)), 
