@@ -11,23 +11,28 @@ want the string in a particular format. */
 
 #include "pmwhdr.h"
 
-#define version    4300      /* In fixed-point format */
-#define COPYRIGHT  US"Copyright (c) Philip Hazel 2018"
+#define version    4310      /* In fixed-point format */
+#define COPYRIGHT  US"Copyright (c) Philip Hazel 2019"
 
 void version_init(void)
 {
+uschar *p;
 uschar today[20];
 copyright = COPYRIGHT;
 Ustrcpy(today, __DATE__);
 if (today[4] == ' ') today[4] = '0';
 today[3] = today[6] = '-';
 version_fixed  = version;
-format_sprintf(version_string, "%f%s (Built ", version,
+p = version_string + format_sprintf(version_string, "%f%s (Built ", version,
   (version%1000 == 0)? ".00" : (version%100 == 0)? "0" : "");
-Ustrncat(version_string, today + 4, 3);
-Ustrncat(version_string, today, 4);
-Ustrncat(version_string, today + 7, 4);
-Ustrcat(version_string, ")");
+
+/* We used to use Ustrncat() to extract and re-arrange the date string, but GCC
+now warns when the whole string isn't copied. */
+
+(void) memcpy(p, today+4, 3);
+(void) memcpy(p+3, today, 4);
+(void) memcpy(p+7, today+7, 4);
+(void) memcpy(p+11, ")\0", 2);
 }
 
 /* End of version.c */
